﻿using APIs.Common;
using APIs.Dto;
using APIs.Req;
using Autofac.Core;
using AutoMapper;
using Common.Utility.Model;
using Common.Utility.RabbitMQ;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ModelBinding;
using Microsoft.CodeAnalysis;
using Microsoft.IdentityModel.Tokens;
using Models.SqlModel;
using Models.Table;
using Models.ToolsModel;
using Newtonsoft.Json;
using Services;
using Services.Interface;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq.Expressions;
using System.Security.Cryptography.Xml;
using static Microsoft.AspNetCore.Razor.Language.TagHelperMetadata;
using static System.Net.Mime.MediaTypeNames;

namespace APIs.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class BaseInfoController : ControllerBase
    {
        public IMapper Mapper { get; }
        private readonly IPoliceService _policeService;
        private readonly IEquipmentTypeService _equipmentTypeService;
        private readonly IEquipmentSizeService _equipmentSizeService;
        private readonly ICarService _carService;
        private readonly IInventoryService _inventoryService;
        private readonly IInvService _invService;
        private readonly IUsersService _usersService;
        private readonly ILogService _logService;
        public BaseInfoController(IMapper mapper,IPoliceService policeService, IEquipmentTypeService equipmentTypeService,
            IEquipmentSizeService equipmentSizeService,ICarService carService, IInventoryService inventoryService,
            IInvService invService, IUsersService usersService, ILogService logService)
        {
            _carService = carService;
            _equipmentSizeService = equipmentSizeService;
            _equipmentTypeService = equipmentTypeService;
            _policeService = policeService;
            _inventoryService = inventoryService;
            _invService = invService;
            _usersService = usersService;
            _logService = logService;
            Mapper = mapper;
        }

        /// <summary>
        /// 3.0平台获取图片
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet]
        public IActionResult DownloadLogPicture(string id)
        {
            try
            {
                // 读取文件内容
                byte[] fileBytes = System.IO.File.ReadAllBytes($"LogPictures\\{id}.jpg");

                // 获取文件的MIME类型
                string contentType = "application/octet-stream"; // 默认使用二进制流的MIME类型，如果需要根据具体文件类型动态获取，请进行适当的处理。

                // 返回文件结果
                return File(fileBytes, contentType, $"{id}.jpg");
            }
            catch (Exception ex)
            {
                return new ContentResult
                {
                    Content = "ERROR:" + ex,
                    ContentType = "text/plain",
                    StatusCode = 400
                };
            }

        }

        /// <summary>
        /// 获取出入库照片
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public async Task<ApiResult> GetLogPicture([FromBody] GetLogPictureReq req)
        {
            try
            {
                //读取图片
                string imageBytes = "";

                string url = $"LogPictures\\{req.logId}.jpg";
                using (FileStream fileStream = new FileStream(url, FileMode.Open))
                {
                    using (BinaryReader reader = new BinaryReader(fileStream))
                    {
                        imageBytes = Convert.ToBase64String(reader.ReadBytes((int)fileStream.Length));
                    }
                }

                var res = new DownloadPicRes() { picture = imageBytes };

                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                    data = res
                };

                return src;
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,
                };
                return error;
            }
        }

        /// <summary>
        /// 获取账号
        /// </summary>
        /// <param name=""></param>
        /// <returns></returns>
        [HttpGet]
        public virtual async Task<ApiResult> GetAccount()
        {
            try
            {
                var accountlist = await _usersService.Query();

                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                    data = accountlist
                };

                return src;
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,
                };
                return error;
            }
        }

        /// <summary>
        /// 手持机出入库记录上传
        /// </summary>
        /// <param name=""></param>
        /// <returns></returns>
        [HttpPost]
        public virtual async Task<ApiResult> UploadHandRecord([FromBody] HandRecordsReq req)
        {
            try
            {
                var rs = await _logService.AddHandLogs(req);

                var src = rs ? new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                } : new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ResultCode.OPERATE_FAILED.Msg,
                };
                return src;
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,
                };
                return error;
            }
        }

        /// <summary>
        /// 盘库记录上传
        /// </summary>
        /// <param name=""></param>
        /// <returns></returns>
        [HttpPost]
        public virtual async Task<ApiResult> UploadInv([FromBody] Models.ReqModel.InvReq req)
        {
            try
            {
                Expression<Func<Inventory, bool>> expression = s => s.lostFlag == "1";
                var inv = await _inventoryService.Query(expression);

                var lostInv = inv?.Select(s => new Inventory()
                {
                    id = s.id,
                    epc = s.epc,
                    typeName = s.typeName,
                    sizeName = s.sizeName,
                    lostFlag = "0",
                    updateTime = DateTime.Now,
                }).ToList();

                var rs = _invService.AddInvLogs(req, lostInv).Result;

                var src = rs ? new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                } : new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ResultCode.OPERATE_FAILED.Msg,
                };
                return src;
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,
                };
                return error;
            }
        }

        /// <summary>
        /// 获取所有装备信息
        /// </summary>
        /// <param name=""></param>
        /// <returns></returns>
        [HttpGet]
        public async Task<ApiResult> GetInventory()
        {
            try
            {
                Expression<Func<Inventory, bool>> expression = s => s.locationType != "1" && !s.bussinessState.Equals("destruction");
                var invlist = await _inventoryService.Query(expression);

                if (invlist == null)
                {
                    var error = new ApiResult
                    {
                        code = ResultCode.OPERATE_FAILED.Code,
                        msg = "库存为空",
                    };
                    return error;
                }

                var baseinv = invlist.Select(s => new
                {
                    typeId = s.typeId,
                    typeName = s.typeName,
                    sizeId = s.sizeId,
                    sizeName = s.sizeName,
                    equipId = s.id,
                    epc = s.epc,
                    state = s.state,
                }).ToList();

                var inv = baseinv.GroupBy(x => new { x.typeId, x.typeName, x.sizeName, x.sizeId })
                    .Select(s => new
                    {
                        typeId = s.Key.typeId,
                        typeName = s.Key.typeName,
                        sizeId = s.Key.sizeId,
                        sizeName = s.Key.sizeName,
                        epcList = baseinv.Where(a => a.sizeId.Equals(s.Key.sizeId))
                        .Select(b => new
                        {
                            equipId = b.equipId,
                            epc = b.epc,
                            state = b.state,
                        }).ToList()
                    }).ToList();

                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                    data = inv
                };
                return src;
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,
                };
                return error;
            }
        }

        /// <summary>
        /// 根据人员id删除照片及照片数据库信息
        /// </summary>
        /// <param name=""></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<ApiResult> DeletePictureById([FromBody] DelPicReq req)
        {
            try
            {
                var police = await _policeService.QueryOne(p => p.id.Equals(req.id));

                if(police == null)
                {
                    var error = new ApiResult
                    {
                        code = ResultCode.POLICE_ERROR.Code,
                        msg = ResultCode.POLICE_ERROR.Msg,
                    };
                    return error;
                }

                string delurl = police.picUrl;

                if (string.IsNullOrEmpty(delurl))
                {
                    var error = new ApiResult
                    {
                        code = ResultCode.OPERATE_FAILED.Code,
                        msg = "照片信息不存在",
                    };
                    return error;
                }

                //删除数据库照片信息
                police.faceInfo = "";
                police.picUrl = "";
                police.picUpdateTime = DateTime.Now;

                bool rs = await _policeService.Update(police);

                if (!rs)
                {
                    var error = new ApiResult
                    {
                        code = ResultCode.OPERATE_FAILED.Code,
                        msg = "照片信息删除失败",
                    };
                    return error;
                }

                //删除照片
                FileInfo mypic = new FileInfo(delurl);
                mypic.Delete();

                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg
                };
                return src;
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,
                };
                return error;
            }
        }

        /// <summary>
        /// 人员照片下载
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public async Task<ApiResult> DownloadPicture([FromForm] DownloadPicReq req)
        {
            try
            {
                var pic = await _policeService.QueryOne(p => !string.IsNullOrEmpty(p.picUrl) && p.picUrl.Equals(req.picUrl));

                //不存在
                if (pic == null)
                {
                    var error = new ApiResult
                    {
                        code = ResultCode.OPERATE_FAILED.Code,
                        msg = "照片不存在",
                    };
                    return error;
                }

                //读取图片
                string imageBytes = "";

                string url = pic.picUrl;
                using (FileStream fileStream = new FileStream(url, FileMode.Open))
                {
                    using (BinaryReader reader = new BinaryReader(fileStream))
                    {
                        imageBytes = Convert.ToBase64String(reader.ReadBytes((int)fileStream.Length));
                    }
                }

                var res = new DownloadPicRes() { picture = imageBytes };

                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                    data = res
                };

                return src;
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,
                };
                return error;
            }
        }

        /// <summary>
        /// 人员照片上传
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public async Task<ApiResult> UploadPicture([FromForm] PictureReq req)
        {
            try
            {
                var police = await _policeService.QueryOne(p => p.id == req.policeId);

                //警员不存在
                if (police == null)
                {
                    var error = new ApiResult
                    {
                        code = ResultCode.POLICE_ERROR.Code,
                        msg = "警员不存在",
                    };
                    return error;
                }

                //存储图片
                var ext = Path.GetExtension(req.picture.FileName).ToLower();

                using (MemoryStream ms = new MemoryStream())
                {
                    //拷贝文件到内存流
                    req.picture.CopyTo(ms);
                    var tmp = ms.GetBuffer();
                    var fileType = tmp[0].ToString() + tmp[1].ToString();

                    var path = "Pictures";
                    var fileName = $"{req.policeId}{Path.GetExtension(req.picture.FileName)}";

                    Directory.CreateDirectory(path);
                    var filePath = Path.Combine(path, fileName);

                    System.Drawing.Image.FromStream(ms).Save(filePath);

                    //更新警员照片地址
                    police.picUrl = filePath;
                    police.updateTime = DateTime.Now;
                    police.picUpdateTime = police.updateTime;
                    bool rs = await _policeService.Update(police);

                    var reply = new UploadPictureReply() { picUrl = filePath, updateTime = (DateTime)police.picUpdateTime }; 

                    var src = rs ? new ApiResult
                    {
                        code = ResultCode.OPERATE_SUCCESS.Code,
                        msg = ResultCode.OPERATE_SUCCESS.Msg,
                        data = reply
                    } : new ApiResult
                    {
                        code = ResultCode.OPERATE_FAILED.Code,
                        msg = "更新照片失败"
                    };

                    return src;

                }
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,
                };
                return error;
            }
        }
        class UploadPictureReply 
        {
            public string picUrl {  get; set; }
            public DateTime updateTime { get; set; }
        }


        /// <summary>
        /// 新增基础信息
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [HttpPost]
        public virtual async Task<ApiResult> UploadInfo([FromBody] InfoListReq req)
        {

            try
            {
                string idlist = string.Join(",", req.infoList.Select(d => d.policeId).ToArray());
                var list = idlist.Split(",");
                var police = await _policeService.Query(p => list.Contains(p.id));
                if (police != null)
                {
                    foreach (var item in police)
                    {
                        var itemnew = req.infoList.Where(p => p.policeId.Equals(item.id)).FirstOrDefault();
                        if (itemnew != null)
                        {
                            item.fingerInfo = itemnew.fingerInfo;
                            item.faceInfo = itemnew.faceInfo;
                        }
                        
                    }
               
                    int result=  _policeService.UpdateObjs(police);
                    if (result > 0)
                    {
                        var src = new ApiResult
                        {
                            code = ResultCode.OPERATE_SUCCESS.Code,
                            msg = ResultCode.OPERATE_SUCCESS.Msg,
                            data = police
                        };
                        return src;
                    }
                    else {
                        var src = new ApiResult
                        {
                            code = ResultCode.POLICE_ERROR.Code,
                            msg = ResultCode.POLICE_ERROR.Msg,
                            data = ""
                        };
                        return src;
                    }
                  
                }
                else
                {
                    var src = new ApiResult
                    {
                        code = ResultCode.POLICE_ERROR.Code,
                        msg = ResultCode.POLICE_ERROR.Msg,
                        data = ""
                    };
                    return src;
                }
                
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,

                };
                return error;
            }

        }

        /// <summary>
        /// 查询人员信息
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [HttpPost]
        public virtual async Task<ApiResult> GetAllPersonnel([FromBody] CommonReq req)
        {

            try
            {
                DateTime reqTime = DateTime.MinValue;
                if (!string.IsNullOrEmpty(req.updateTime))
                {
                    reqTime = DateTime.Parse(req.updateTime);
                }
                var police = await _policeService.Query(p=>p.updateTime>= reqTime);
                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                    data = police
                };
                return src;
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,

                };
                return error;
            }

        }

        /// <summary>
        /// 查询装备类型信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]

        public  async Task<string> GetEquipmentType([FromBody] CommonReq req)
        {

            try
            {
                DateTime reqTime = DateTime.MinValue;
                if (!string.IsNullOrEmpty(req.updateTime))
                {
                    reqTime = DateTime.Parse(req.updateTime);
                }
                var typeQuery = await _equipmentTypeService.Query(p => p.updateTime >= reqTime);
                List<EquipmentTypeDto> typeDto = new List<EquipmentTypeDto>() ;
                Mapper.Map(typeQuery, typeDto);
                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                    data= typeDto
                };
                return JsonConvert.SerializeObject(src);
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,
                  
                };
                return JsonConvert.SerializeObject(error);
            }

        }

        /// <summary>
        /// 查询装备号型信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]

        public async Task<string> GetEquipmentSize([FromBody] CommonReq req)
        {

            try
            {
                DateTime reqTime= DateTime.MinValue;
                if (!string.IsNullOrEmpty(req.updateTime))
                {
                    reqTime = DateTime.Parse(req.updateTime);
                }
                var sizeQuery = await _equipmentSizeService.Query(p => p.updateTime >= reqTime);
                List<EquipmentSizeDto> sizeDto = new List<EquipmentSizeDto>();
                Mapper.Map(sizeQuery, sizeDto);
                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                    data = sizeDto
                };
               
                return JsonConvert.SerializeObject(src);
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,

                };
                return JsonConvert.SerializeObject(error); 
            }

        }
        /// <summary>
        /// 查询车辆信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]

        public async Task<string> GetCar([FromBody] CommonReq req)
        {

            try
            {
                DateTime reqTime = DateTime.MinValue;
                if (!string.IsNullOrEmpty(req.updateTime))
                {
                    reqTime = DateTime.Parse(req.updateTime);
                }
                var carQuery = await _carService.Query(p => p.updateTime >= reqTime);
                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                    data = carQuery
                };

                return JsonConvert.SerializeObject(src);
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,

                };
                return JsonConvert.SerializeObject(error);
            }

        }

        /// <summary>
        /// 查询当前时间信息
        /// </summary>
        /// <returns></returns>
        [HttpGet]

        public async Task<string> GetTime()
        {
            try
            {

                var src = new ApiResult
                {
                    code = ResultCode.OPERATE_SUCCESS.Code,
                    msg = ResultCode.OPERATE_SUCCESS.Msg,
                    data = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")
            };

                return JsonConvert.SerializeObject(src);
            }
            catch (Exception ex)
            {
                var error = new ApiResult
                {
                    code = ResultCode.OPERATE_FAILED.Code,
                    msg = ex.Message,

                };
                return JsonConvert.SerializeObject(error);
            }
         
        }
        /// <summary>
        /// 发送消息队列测试
        /// </summary>
        /// <returns></returns>
        [HttpPost]

        public async Task<string> SendMQ([FromBody] RecordsReq req)
        {
            
            await RabbitMQContext.SendMessageAsynce(JsonConvert.SerializeObject(req));
            var src = new ApiResult
            {
                code = ResultCode.OPERATE_SUCCESS.Code,
                msg = ResultCode.OPERATE_SUCCESS.Msg
                
            };
            return JsonConvert.SerializeObject(src);
        }
    }
}
